# 사용자 질의 의도 분석

## About The Project

* 영화 추천을 위해 사용자 질의의 의도 파악이 필요

## Built With
프로젝트를 부트스트랩하는 데 사용되는 주요 프레임워크/라이브러리가 나열. 

* python >= 3.9
* tensorflow
* numpy
* pandas
* scikit-learn
* keras
* konlpy

<!-- GETTING STARTED -->
## Getting Started
### 사용자 정의 사전 생성 (dict 폴더)

* Fullmeta 필드를 레이블 별로 분류한 후 레이블 별 사전을 생성
* 사용자 사전 파일을 하나의 txt 파일로 생성

> 실행 방법

    python dup_dict.py "사용자 사전 파일 1"
    python dup_dict.py "사용자 사전 파일 2"
    python dup_dict.py "사용자 사전 파일 3"

> output

    user_dic.txt
    이 때, user_dic.txt 내용은 사용자 사전 파일 1,2,3의 정보를 하나로 합친 내용임

---

### 단어 인덱스 사전 생성 (train_tools 폴더)

* 학습할 데이터 셋의 단어 인덱스 사전을 생성
* 학습할 데이터 셋은 네이버 영화 리뷰 긍/부정 데이터이며 해당 데이터를 형태소 분석하여 단어 인덱스 사전을 생성
* 단어 인덱스 사전 생성 시 제외한 형태소는 다음과 같다.
    * 주격조사, 보격조사, 관형격조사, 목적격조사, 부사격조사, 호격조사, 인용격조사 ('JKS', 'JKC', 'JKG', 'JKO', 'JKB', 'JKV', 'JKQ')
    * 보조사, 접속조사 ('JX', 'JC')
    * 마침표,물음표,느낌표(SF), 쉼표,가운뎃점,콜론,빗금(SP), 따옴표,괄호표,줄표(SS), 줄임표(SE), 붙임표(물결,숨김,빠짐)(SO)
    * 선어말어미, 종결어미, 연결어미, 명사형전성어미, 관형형전성어미 ('EP', 'EF', 'EC', 'ETN', 'ETM')
    * 명사파생접미사, 동사파생접미사, 형용사파생접미사 ('XSN', 'XSV', 'XSA')

> 실행 방법

    python create_dict.py

> output

    chatbot_dict.bin

---

### CNN 기반 의도 분석

* 데이터 셋을 의도별로 분류한 뒤 1D CNN 학습한 정보를 기반으로 입력 받은 질의에 대한 의도 분석을 진행
* 진행 순서는 다음과 같음

1. 사전 정보 저장
    * Fullmeta에서 추출한 정보를 기반으로 생성한 5개의 사전 정보(인물, 클래스, 장르, 소재, 감성) 를 불러오기

2. 의도 분석에 사용할 데이터 파일 다운로드
    * 해당 데이터 파일은 네이버 영화 리뷰 긍/부정 데이터이며 ratings_train 파일에는 15만개, rating_test 파일에는 5만개의 리뷰가 포함되어 있음
    * 의도 분석에 사용하기 위해 document 내용을 추출하여 사용함


    > 데이터 파일 구조

        [id] [\t] [document] [\t] [label]

    > 예

        9976970 아 더빙.. 진짜 짜증나네요 목소리 0
        3819312 흠...포스터보고 초딩영화줄....오버연기조차 가볍지 않구나 1
        10265843 너무재밓었다그래서보는것을추천한다 0
        9045019 교도소 이야기구먼 ..솔직히 재미는 없다..평점 조정 0
        6483659 사이몬페그의 익살스런 연기가 돋보였던 영화!스파이더맨에서 늙어보이기만 했던 커스틴 던스트가 너무나도 이뻐보였다 1

3. 문장 별 레이블 설정
    * 분류한 문장에 레이블을 붙여 해당 문장이 어떤 정보와 연관되는지 파악 가능
    * 해당 문장에 사전(인물, 클래스, 장르, 소재, 감정)에 포함된 어휘가 존재한다면 분류

4. 데이터 전처리 및 시퀀스 벡터 생성
    * 분류한 문장에 대하여 전처리 작업 진행 후 CNN 학습을 위한 시퀀스 벡터를 생성
    * 이 때, 단어 시퀀스 벡터의 크기를 동일하게 맞추기 위하여 MAX_SEQ_LEN 크기만큼 시퀀스 벡터의 패딩 처리를 진행

5. 데이터 셋 분류
    * 패딩 처리된 시퀀스 벡터 리스트와 레이블 정보를 tensor 객체로 만든 후 랜덤으로 섞어 학습용, 검증용, 테스트용 데이터 셋을 생성

6. CNN 모델 기반 의도 분류 모델 구현
    * 케라스 함수형 모델 방식으로 의도 분류 모델 구현
    * Hyperparameter 설정은 다음과 같다.
        * 임베딩 벡터 차원: 128
        * 드롭 아웃 비율: 0.5
        * 커널 수: 128
        * 커널 크기: 3, 4, 5
        * 은닉층 뉴런 수: 128
        * 활성화 함수: softmax 함수

7. CNN 모델 학습 및 문장 분류
    * CNN 모델 생성 후 컴파일 함수를 통해 CNN 모델 컴파일 진행
        * 최적화 방법: adam
        * 손실 함수: sparse_categorical_crossentropy
    * EPOCH 수 만큼 모델 학습 진행 후 성능 평가
    * 학습 완료된 모델은 keras 파일 포멧으로 저장
    * 문장을 입력 받아 학습 완료된 모델에 입력하면 해당 문장과 연관된 레이블을 붙여줌


    > 예1

        질의: 송강호 나오는거 추천
        예측 결과: [[9.7998375e-01 7.5950357e-03 2.5250551e-03 9.7950809e-03 1.0117089e-04]]
        의도 예측 클래스 :  0
        의도 예측 레이블 :  character

    > 예2

        질의: 애니메이션 추천해줘
        예측 결과: [[8.9980122e-03 9.4120288e-01 4.3777255e-03 4.5190416e-02 2.3104453e-04]]
        의도 예측 클래스 :  1
        의도 예측 레이블 :  class

8. 레이블 별 질의 생성
    * 사용자 질의의 단어 시퀀스와 사전 간의 매칭을 통해 개체명 인식 진행
    * 개체명 인식 내용과 레이블 관련 필드를 이용하여 레이블 별 질의를 생성


    > 예

        질의: 액션 추천해줘
        개체명 인식: [{'word': '액션', 'label': 'genre'}]
        질의 생성 결과: (has장르:"액션")

* 소스코드는 아래 파일을 통해 확인 가능하다.

    intent_classification.ipynb


